#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

from zope.component import adapter
from zope.container.contained import NameChooser
from zope.container.interfaces import INameChooser
from zope.interface import implementer, implements
from zope.security.proxy import removeSecurityProxy

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.course.interfaces import ICourse
from schooltool.courseinfo.courseinfo import (COURSEINFO_ROOT_KEY,
    CourseInfoSchoolYear, CurrentCourseInfoContainer, CurrentCourseInfo,
    CourseInfoRevision, PublishedCourseInfoContainer, CourseInfoRevisionContainer,
    URICurrentCourseInfo, URICourseInfoRevision)
from schooltool.relationship.relationship import getRelatedObjects
from schooltool.schoolyear.interfaces import ISchoolYear, ISchoolYearContainer

import interfaces


@adapter(ISchoolToolApplication)
@implementer(interfaces.ICourseInfoRoot)
def getCourseInfoRoot(app):
    if COURSEINFO_ROOT_KEY not in app:
        return None
    return app[COURSEINFO_ROOT_KEY]


@adapter(ISchoolYear)
@implementer(interfaces.ICourseInfoSchoolYear)
def getCourseInfoSchoolYear(year):
    root = interfaces.ICourseInfoRoot(ISchoolToolApplication(None))
    if year.__name__ not in root:
        obj = CourseInfoSchoolYear()
        root[year.__name__] = obj
        obj['current_courseinfos'] = CurrentCourseInfoContainer()
        obj['published_courseinfos'] = PublishedCourseInfoContainer()
        obj['courseinfo_revisions'] = CourseInfoRevisionContainer()
    return root[year.__name__]


@adapter(interfaces.ICourseInfoSchoolYear)
@implementer(ISchoolYear)
def getSchoolYear(courseinfoyear):
    app = ISchoolToolApplication(None)
    schoolyears = ISchoolYearContainer(app)
    return schoolyears[courseinfoyear.__name__]


@adapter(interfaces.ICourseInfo)
@implementer(interfaces.ICourseInfoSchoolYear)
def getCourseInfoSchoolYearFromCourseInfo(courseinfo):
    return courseinfo.__parent__.__parent__


@adapter(interfaces.ICourseInfoSchoolYear)
@implementer(interfaces.ICurrentCourseInfoContainer)
def getCurrentCourseInfoContainer(year):
    return year['current_courseinfos']


@adapter(interfaces.ICourseInfoSchoolYear)
@implementer(interfaces.IPublishedCourseInfoContainer)
def getPublishedCourseInfoContainer(year):
    return year['published_courseinfos']


@adapter(interfaces.ICourseInfoSchoolYear)
@implementer(interfaces.ICourseInfoRevisionContainer)
def getCourseInfoRevisionContainer(year):
    return year['courseinfo_revisions']


@adapter(interfaces.ICurrentCourseInfo)
@implementer(ICourse)
def getCurrentCourseInfoCourse(courseinfo):
    return [x for x in courseinfo.courses][0]


@adapter(ICourse)
@implementer(interfaces.ICurrentCourseInfo)
def getCurrentCourseInfo(course):
    courseinfos = getRelatedObjects(course, URICurrentCourseInfo)
    if not courseinfos:
        # Auto-vivification
        courseinfo = CurrentCourseInfo()
        courseinfo.courses.add(removeSecurityProxy(course))

        year = interfaces.ICourseInfoSchoolYear(ISchoolYear(course))
        container = interfaces.ICurrentCourseInfoContainer(year)

        chooser = INameChooser(container)
        name = chooser.chooseName('', courseinfo)
        container[name] = courseinfo

        courseinfos = [courseinfo]
    return courseinfos[0]


@adapter(interfaces.ICurrentCourseInfo)
@implementer(interfaces.IPublishedCourseInfo)
def getPublishedCourseInfoFromCurrent(current):
    year = interfaces.ICourseInfoSchoolYear(current)
    published_courses = interfaces.IPublishedCourseInfoContainer(year)
    for published_course in published_courses.values():
        if published_course.course_id == current.course_id:
            return published_course
    return None

@adapter(interfaces.IPublishedCourseInfo)
@implementer(interfaces.ICurrentCourseInfo)
def getCurrentCourseInfoFromPublished(published):
    year = interfaces.ICourseInfoSchoolYear(published)
    current_courses = interfaces.ICurrentCourseInfoContainer(year)
    for current_course in current_courses.values():
        if current_course.course_id == published.course_id:
            return current_course
    return None
    

@adapter(ICourse)
@implementer(interfaces.IPublishedCourseInfo)
def getPublishedCourseInfoFromCourse(course):
    currentcourseinfo = interfaces.ICurrentCourseInfo(course)
    return interfaces.IPublishedCourseInfo(currentcourseinfo)


@adapter(interfaces.ICurrentCourseInfoContainer)
@implementer(INameChooser)
class CourseInfoNameChooser(NameChooser):

    implements(INameChooser)

    def chooseName(self, name, obj):
        """See INameChooser."""

        i = 1
        n = "1"
        while n in self.context:
            i += 1
            n = unicode(i)
        # Make sure the name is valid
        self.checkName(n, obj)
        return n

@adapter(interfaces.IPublishedCourseInfo)
@implementer(interfaces.ICourseInfoRevision)
def getCourseInfoRevisionFromPublished(published):
    year = interfaces.ICourseInfoSchoolYear(published)
    revisions = interfaces.ICourseInfoRevisionContainer(year)
    for revision in revisions.values():
        if revision.course_id == published.course_id:
            return revision
    # Auto-vivification
    courseinfo = CourseInfoRevision(published.title)
    courseinfo.course_id = published.course_id

    year = interfaces.ICourseInfoSchoolYear(published)
    container = interfaces.ICourseInfoRevisionContainer(year)
    chooser = INameChooser(container)
    name = chooser.chooseName('', courseinfo)
    container[name] = courseinfo

    return courseinfo

@adapter(interfaces.ICourseInfoRevision)
@implementer(interfaces.IPublishedCourseInfo)
def getPublishedCourseInfoFromRevision(revision):
    year = interfaces.ICourseInfoSchoolYear(revision)
    publisheds = interfaces.IPublishedCourseInfoContainer(year)
    for published in publisheds.values():
        if published.course_id == revision.course_id:
            return published
    return None

