#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
Forms for CourseInfo
"""


from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
from zope.component import adapts
from zope.container.interfaces import INameChooser
from zope.publisher.browser import BrowserView
from zope.traversing.browser.absoluteurl import absoluteURL

from z3c.form import form, field, button
from z3c.form.converter import BaseDataConverter
from z3c.form.interfaces import ITextAreaWidget, DISPLAY_MODE

from schooltool.course.interfaces import ICourse
from schooltool.courseinfo import interfaces
from schooltool.courseinfo import CourseinfoMessage as _
from schooltool.courseinfo.courseinfo import PublishedCourseInfo, CourseInfoRevision


class CustomListDataConverter(BaseDataConverter):
    """A special data converter for iso enums."""

    adapts(interfaces.CourseInfoValueList, ITextAreaWidget)

    def toWidgetValue(self, value):
        """See interfaces.IDataConverter"""
        if value is self.field.missing_value:
            return u''
        return unicode('\n'.join(value))

    def toFieldValue(self, value):
        """See interfaces.IDataConverter"""
        if value == u'':
            return self.field.missing_value
        lines = filter(None, [s.strip() for s in unicode(value).splitlines()])
        if not lines:
            return self.field.missing_value
        field_value = []
        for line in lines:
            field_value.append(line)
        return field_value


class CurrentCourseInfoEditView(form.EditForm):
    """A view for adding a new courseinfo."""
    form.extends(form.EditForm)
    template = ViewPageTemplateFile('templates/add_edit_form.pt')

    fields = field.Fields(interfaces.ICurrentCourseInfo)
    fields = fields.omit('__name__', '__parent__')

    def updateActions(self):
        super(CurrentCourseInfoEditView, self).updateActions()
        self.actions['apply'].addClass('button-ok')
        self.actions['cancel'].addClass('button-cancel')

    @button.buttonAndHandler(_("Cancel"))
    def handle_cancel_action(self, action):
        self.request.response.redirect(self.nextURL())

    def nextURL(self):
        url = absoluteURL(self.context, self.request)
        return url

    @property
    def label(self):
        return _(u'Editing advanced course information for ${course_title}',
                 mapping={'course_title': self.context.title})


class CurrentCourseInfoView(form.DisplayForm):
    """Display form for basic contact."""
    template = ViewPageTemplateFile('templates/display_form.pt')
    fields = field.Fields(interfaces.ICurrentCourseInfo)
    fields = fields.omit('__name__', '__parent__')

    def __init__(self, context, request):
        super(CurrentCourseInfoView, self).__init__(context, request)
        course = ICourse(self.context)
        course_return_url = absoluteURL(course, request)

        self.return_urls = [{'url':course_return_url, 'text': _('Back to course')}]

        published_course = None
        try:
            published_course = interfaces.IPublishedCourseInfo(self.context)
        except TypeError:
            pass

        transfer_info_url = absoluteURL(context, request) + "/infotransfer.html"

        if published_course:
            url = absoluteURL(published_course, request)
            self.return_urls.append({'url': url, 'text': _('This course in the Program of Studies')})
            self.return_urls.append({'url':  transfer_info_url,
                                     'text': _('Overwrite Program of Studies for this course using current info')})
        else:
            self.return_urls.append({'url':  transfer_info_url,
                                     'text': _('Create Program of Studies entry for this course using current info')})

    def __call__(self):
        self.update()
        return self.render()


class CurrentCourseInfoTransferToPublishedView(BrowserView):
    def __init__(self, context, request):
        super(CurrentCourseInfoTransferToPublishedView, self).__init__(context, request)
        try:
            self.published_course = interfaces.IPublishedCourseInfo(self.context)
        except TypeError:
            container = interfaces.IPublishedCourseInfoContainer(
                interfaces.ICourseInfoSchoolYear(self.context))

            self.published_course = PublishedCourseInfo(self.context.title)
            chooser = INameChooser(container)
            name = chooser.chooseName('', self.published_course)
            container[name] = self.published_course

        from zope.security.proxy import removeSecurityProxy as _
        # Necessary to transfer some objects, like lists
            
        self.published_course.course_id = _(self.context.course_id)
        self.published_course.government_id = _(self.context.government_id)
        self.published_course.title = _(self.context.title)
        self.published_course.description = _(self.context.description)
        self.published_course.start_grade = _(self.context.start_grade)
        self.published_course.end_grade = _(self.context.end_grade)
        self.published_course.duration = _(self.context.duration)
        self.published_course.credits = _(self.context.credits)
        self.published_course.elective = _(self.context.elective)
        self.published_course.division = _(self.context.division)
        self.published_course.cluster = _(self.context.cluster)
        self.published_course.pathway = _(self.context.pathway)
        self.published_course.prerequisite = _(self.context.prerequisite)
        self.published_course.credit_codes = _(self.context.credit_codes)
        self.published_course.supervisor = _(self.context.supervisor)
        self.published_course.weight = _(self.context.weight)
        self.published_course.lab_fee = _(self.context.lab_fee)
        self.published_course.requirements = _(self.context.requirements)
        self.published_course.course_description = _(self.context.course_description)
        self.published_course.outcomes = _(self.context.outcomes)
        self.published_course.notes = _(self.context.notes)


    def __call__(self):
        self.request.response.redirect(absoluteURL(self.published_course, self.request))
        
class PublishedCourseInfoEditView(form.EditForm):
    """A view for adding a new courseinfo."""
    form.extends(form.EditForm)
    template = ViewPageTemplateFile('templates/add_edit_form.pt')

    fields = field.Fields(interfaces.IPublishedCourseInfo)
    fields = fields.omit('__name__', '__parent__')

    def updateActions(self):
        super(PublishedCourseInfoEditView, self).updateActions()
        self.actions['apply'].addClass('button-ok')
        self.actions['cancel'].addClass('button-cancel')

    def updateWidgets(self):
        super(PublishedCourseInfoEditView, self).updateWidgets()
        self.widgets['course_id'].mode = DISPLAY_MODE
        self.widgets['course_id'].required = False

    @button.buttonAndHandler(_("Cancel"))
    def handle_cancel_action(self, action):
        self.request.response.redirect(self.nextURL())

    def nextURL(self):
        url = absoluteURL(self.context, self.request)
        return url

    @property
    def label(self):
        return _(u'Editing advanced course information for ${course_title}',
                 mapping={'course_title': self.context.title})


class PublishedCourseInfoView(form.DisplayForm):
    """Display form for basic contact."""
    template = ViewPageTemplateFile('templates/display_form.pt')
    fields = field.Fields(interfaces.IPublishedCourseInfo)
    fields = fields.omit('__name__', '__parent__')

    def __init__(self, context, request):
        super(PublishedCourseInfoView, self).__init__(context, request)
        self.return_urls = []
        try:
            course = ICourse(self.context)
            course_return_url = absoluteURL(course, request)
            self.return_urls.append({'url': course_return_url, 'text': _('Back to course')})
        except TypeError:
            pass

        course_container_url = absoluteURL(self.context.__parent__, request)
        self.return_urls.append({'url': course_container_url, 'text': _('Back to program of studies')})

    def __call__(self):
        self.update()
        return self.render()

class CourseInfoRevisionAddView(form.AddForm):
    """A view for adding a new courseinfo."""
    form.extends(form.AddForm)
    template = ViewPageTemplateFile('templates/add_edit_form.pt')

    fields = field.Fields(interfaces.ICourseInfoRevision)
    fields = fields.omit('__name__', '__parent__', 'deletion', 'creation')

    def create(self, data):
        courseinfo = CourseInfoRevision(data['title'])
        courseinfo.creation = True
        form.applyChanges(self, courseinfo, data)
        return courseinfo

    def add(self, courseinfo):
        name = INameChooser(self.context).chooseName('', courseinfo)
        self.context[name] = courseinfo

    def updateActions(self):
        super(CourseInfoRevisionAddView, self).updateActions()
        self.actions['add'].addClass('button-ok')
        self.actions['cancel'].addClass('button-cancel')

    @button.buttonAndHandler(_("Cancel"))
    def handle_cancel_action(self, action):
        self.request.response.redirect(self.nextURL())

    def nextURL(self):
        url = absoluteURL(self.context, self.request)
        return url

    @property
    def label(self):
        return _(u'Editing advanced course information for ${course_title}',
                 mapping={'course_title': self.context.title})

class CourseInfoRevisionEditView(form.EditForm):
    """A view for adding a new courseinfo."""
    form.extends(form.EditForm)
    template = ViewPageTemplateFile('templates/add_edit_form.pt')

    fields = field.Fields(interfaces.ICourseInfoRevision)
    fields = fields.omit('__name__', '__parent__', 'creation')

    def __init__(self, context, request):
        super(CourseInfoRevisionEditView, self).__init__(context, request)

        if self.context.creation:
            self.fields = self.fields.omit('deletion')

        try:
            interfaces.IPublishedCourseInfo(self.context)
            self.request.response.redirect(absoluteURL(self.context, request)+"/compareedit.html")
        except TypeError:
            pass

    def updateActions(self):
        super(CourseInfoRevisionEditView, self).updateActions()
        self.actions['apply'].addClass('button-ok')
        self.actions['cancel'].addClass('button-cancel')

    @button.buttonAndHandler(_("Cancel"))
    def handle_cancel_action(self, action):
        self.request.response.redirect(self.nextURL())

    def nextURL(self):
        url = absoluteURL(self.context, self.request)
        return url

    @property
    def label(self):
        return _(u'Editing course info revision proposal')


class CourseInfoRevisionCompareEditView(BrowserView):
    """A view for adding a new courseinfo."""

    def __init__(self, context, request):
        super(CourseInfoRevisionCompareEditView, self).__init__(context, request)

        self.published_courseinfo = interfaces.IPublishedCourseInfo(self.context)

    def textify(self, textlist):
        if not textlist:
            return ''
        if type(textlist)==str or type(textlist)==unicode:
            return textlist
        return '\n'.join(textlist)
        
    def handle_cancel_action(self):
        self.request.response.redirect(self.nextURL())

    def nextURL(self):
        url = absoluteURL(self.context, self.request)
        return url


class CourseInfoRevisionView(form.DisplayForm):
    """Display form for basic contact."""
    template = ViewPageTemplateFile('templates/display_form.pt')
    fields = field.Fields(interfaces.IPublishedCourseInfo)
    fields = fields.omit('__name__', '__parent__', 'creation', 'deletion')

    def __init__(self, context, request):
        super(CourseInfoRevisionView, self).__init__(context, request)
        self.return_urls = []

        course_container_url = absoluteURL(self.context.__parent__, request)
        self.return_urls.append({'url': course_container_url, 'text': _(u'Back to list of course revisions')})

        try:
            publishedcourse = interfaces.IPublishedCourseInfo(self.context)
            course_return_url = absoluteURL(publishedcourse, request)
            self.return_urls.append({'url': course_return_url, 'text': _('This course in the Program of Studies')})
        except TypeError:
            pass

    def __call__(self):
        self.update()
        return self.render()
