#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
Implementations of CourseInfo and associated objects.
"""

from persistent import Persistent

from zope.annotation.interfaces import IAttributeAnnotatable
from zope.container.btree import BTreeContainer
from zope.container.contained import Contained
from zope.interface import implements

from schooltool.app.relationships import URICourse
from schooltool.app.app import InitBase, StartUpBase
from schooltool.course.interfaces import ICourse
from schooltool.relationship import RelationshipProperty
from schooltool.relationship.uri import URIObject

import interfaces


COURSEINFO_ROOT_KEY = 'schooltool.courseinfo'


URICurrentCourseInfoCourses = URIObject(
    'http://schooltool.org/ns/courseinfo/currentcourseinfocourses',
    'CurrentCourseInfoCourses', 'The courses linked to a CurrentCourseInfo.')
URIPublishedCourseInfoCourses = URIObject(
    'http://schooltool.org/ns/courseinfo/publishedcourseinfocourses',
    'PublishedCourseInfoCourses',
    'The courses linked to a PublishedCourseInfo.')
URICourseInfoRevisionCourses = URIObject(
    'http://schooltool.org/ns/courseinfo/courseinforevisioncourses',
    'CourseInfoRevisionCourses', 'The courses linked to a CourseInfoRevision.')
URICurrentCourseInfo = URIObject(
    'http://schooltool.org/ns/courseinfo/currentcourseinfo',
    'CurrentCourseInfo', 'The CurrentCourseInfo linked to a course.')
URIPublishedCourseInfo = URIObject(
    'http://schooltool.org/ns/courseinfo/publishedcourseinfo',
    'PublishedCourseInfo', 'The PublishedCourseInfo linked to a course.')
URICourseInfoRevision = URIObject(
    'http://schooltool.org/ns/courseinfo/courseinforevision',
    'CourseInfoRevision', 'The CourseInfoRevision linked to a course.')


class CourseInfoRoot(BTreeContainer):
    """Root container of the schooltool.courseinfo."""

    implements(interfaces.ICourseInfoRoot)


class CourseInfoSchoolYear(BTreeContainer):
    """Container of CourseInfo containers."""

    implements(interfaces.ICourseInfoSchoolYear)


class CurrentCourseInfoContainer(BTreeContainer):
    """Container of CourseInfo objects."""

    implements(interfaces.ICurrentCourseInfoContainer)


class PublishedCourseInfoContainer(BTreeContainer):
    """Container of the current year's CourseInfo objects."""

    implements(interfaces.IPublishedCourseInfoContainer)


class CourseInfoRevisionContainer(BTreeContainer):
    """Container of the next year's (proposed) CourseInfo objects."""

    implements(interfaces.ICourseInfoRevisionContainer)


class CourseInfo(Persistent, Contained):
    """The main course info object."""

    course_id = None
    title = None
    description = None
    start_grade = None
    end_grade = None
    duration = None
    credits = None
    elective = None
    division = None
    cluster = None
    pathway = None
    requirements = None
    government_id = None
    credit_codes = None
    supervisor = None
    planning_information = None
    course_description = None
    weight = None
    lab_fee = None
    notes = None
    prerequisite = None
    outcomes = None

    def __init__(self, title, *args, **kwargs):
        if title:
            self.title = title
        super(CourseInfo, self).__init__(*args, **kwargs)

    def __repr__(self):
        return '<%s.%s (%r)>' % (self.__class__.__module__,
                                 self.__class__.__name__,
                                 self.title)


class CurrentCourseInfo(CourseInfo):
    """The course info object for a course in the current year. """

    implements(interfaces.ICurrentCourseInfo, IAttributeAnnotatable)

    courses = RelationshipProperty(URICurrentCourseInfoCourses,
                                   URICurrentCourseInfo,
                                   URICourse)

    def __init__(self, *args, **kwargs):
        super(CurrentCourseInfo, self).__init__('', *args, **kwargs)

    def _get_course_id(self):
        return ICourse(self).course_id if ICourse(self) else ''

    def _set_course_id(self, value):
        if ICourse(self):
            ICourse(self).course_id = value
    course_id = property(_get_course_id, _set_course_id)

    def _get_title(self):
        return ICourse(self).title if ICourse(self) else ''

    def _set_title(self, value):
        if ICourse(self):
            ICourse(self).title = value
    title = property(_get_title, _set_title)

    def _get_description(self):
        return ICourse(self).description if ICourse(self) else ''

    def _set_description(self, value):
        if ICourse(self):
            ICourse(self).description = value
    description = property(_get_description, _set_description)

    def _get_credits(self):
        return ICourse(self).credits if ICourse(self) else ''

    def _set_credits(self, value):
        if ICourse(self):
            ICourse(self).credits = value
    credits = property(_get_credits, _set_credits)

    def _get_government_id(self):
        return ICourse(self).government_id if ICourse(self) else ''

    def _set_government_id(self, value):
        if ICourse(self):
            ICourse(self).government_id = value
    government_id = property(_get_government_id, _set_government_id)


class PublishedCourseInfo(CourseInfo):
    """The course info object for a course published in the program of studies
       catalog for next year. """

    implements(interfaces.IPublishedCourseInfo, IAttributeAnnotatable)

    courses = RelationshipProperty(URIPublishedCourseInfoCourses,
                                   URIPublishedCourseInfo,
                                   URICourse)


class CourseInfoRevision(CourseInfo):
    """The course info object for a revision to the program of studies."""

    implements(interfaces.ICourseInfoRevision, IAttributeAnnotatable)

    courses = RelationshipProperty(URICourseInfoRevisionCourses,
                                   URICourseInfoRevision,
                                   URICourse)
    creation = False
    deletion = False


def setUpCourseInfo(app):
    app[COURSEINFO_ROOT_KEY] = CourseInfoRoot()


class CourseInfoInit(InitBase):
    """Create the CourseInfoRoot object."""

    def __call__(self):
        setUpCourseInfo(self.app)


class CourseInfoStartup(StartUpBase):
    def __call__(self):
        if COURSEINFO_ROOT_KEY not in self.app:
            setUpCourseInfo(self.app)
