#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
Unit tests for course and section implementations.
"""
from zope.container.interfaces import IContainer, IContained
from zope.container.constraints import contains, containers
from zope.interface import Attribute
from zope.schema import TextLine, Text, Int, Dict, List, Date, Float, Bool

from schooltool.courseinfo import CourseinfoMessage as _


class ICourseInfoRoot(IContainer):
    """The interface for the root container of schooltool.courseinfo."""


class ICourseInfoSchoolYear(IContainer):
    """The interface for the CourseInfoSchoolYear container."""

    containers(ICourseInfoRoot)


class ICurrentCourseInfoContainer(IContainer):
    """The interface for the container of courseinfo objects."""

    contains('schooltool.courseinfo.interfaces.ICurrentCourseInfo')
    containers(ICourseInfoSchoolYear)


class IPublishedCourseInfoContainer(IContainer):
    """The interface for a container for published courseinfo
       (program of studies) objects."""

    contains('schooltool.courseinfo.interfaces.IPublishedCourseInfo')
    containers(ICourseInfoSchoolYear)


class ICourseInfoRevisionContainer(IContainer):
    """The interface for a container of revisions to the program of studies."""

    contains('schooltool.courseinfo.interfaces.ICourseInfoRevision')
    containers(ICourseInfoSchoolYear)


class CourseInfoValueList(List):
    """Marker field for course info list fields."""


class ICourseInfo(IContained):
    """The interface for the courseinfo object."""

    course_id = TextLine(
        title=u"Local Course ID",
        required=True,
        )

    government_id = TextLine(
        title=u"Government ID",
        required=False,
        )

    title = TextLine(
        title=u"Course title",
        )

    description = Text(
        title=u"Local Title",
        required=False,
        )

    start_grade = Int(
        title=u"Start grade",
        required=False,
        )

    end_grade = Int(
        title=u"End grade",
        required=False,
        )

    duration = Int(
        title=u"Duration",
        description=u"In weeks",
        required=False,
        )

    credits = Float(
        title=u"Credits",
        required=False,
        )

    elective = Bool(
        title=u"Elective",
        required=False,
        )

    division = TextLine(
        title=u"Division",
        required=False,
        )

    cluster = TextLine(
        title=u"Cluster",
        required=False,
        )

    pathway = TextLine(
        title=u"Pathway",
        required=False,
        )

    prerequisite = CourseInfoValueList(
        title=_("Prerequisites"),
        value_type=TextLine(title=u"Prerequisite"),
        required=False,
        description=_("One local course ID per line"),
        )

    credit_codes = CourseInfoValueList(
        title=_("Credit codes"),
        value_type=TextLine(title=u"Credit code"),
        required=False,
        )

    supervisor = TextLine(
        title=u"Supervisor",
        required=False,
        )

    weight = Float(
        title=u"Weight",
        required=False,
        )

    lab_fee = Float(
        title=u"Lab fee",
        required=False,
        )

    requirements = Text(
        title=_("Requirements"),
        required=False,
        )

    course_description = CourseInfoValueList(
        title=_("Course Description"),
        value_type=TextLine(title=u"Course description line"),
        required=False,
        )

    outcomes = Text(
        title=u"Outcomes",
        required=False,
        )

    notes = Text(
        title=u"Notes",
        required=False,
        )

    courses = Attribute(
        """A list of courses linked to this couseinfo object.""")


class ICurrentCourseInfo(ICourseInfo):
    """ The interface for courseinfo of a course in the current schoolyear."""

    containers(ICurrentCourseInfoContainer)


class IPublishedCourseInfo(ICourseInfo):
    """ The interface for courseinfo published in the current year's program
        of studies."""

    containers(IPublishedCourseInfoContainer)


class ICourseInfoRevision(ICourseInfo):
    """ The interface for storing a revision to courseinfo in the program of
        studies. """

    containers(ICourseInfoRevisionContainer)

    deletion = Bool(
        title=u"Propose deletion of this course",
        required=False,
        default=False,
        )

    # Backend field only, used for display and tracking purposes
    creation = Bool(
        title=u"New Course Proposition",
        required=False,
        default=False,
        )
